@@ -25,13 +25,32 @@
 #include <libxml/xmlsave.h>
 
 #include "common.h"
-
-/* %TRUE if there's no Internet connection, so we should only run local tests */
-static gboolean no_internet = FALSE;
+#include "mock-server.h"
 
 /* %TRUE if interactive tests should be skipped because we're running automatically (for example) */
 static gboolean no_interactive = FALSE;
 
+/* declaration of debug handler */
+static void gdata_test_debug_handler (const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data);
+
+/* declaration of printerr handler */
+static void gdata_test_assert_handler (const gchar *message);
+
+/* global list of debugging messages */
+static GSList *message_list = NULL;
+
+/* Directory to output network trace files to, if trace output is enabled. (NULL otherwise.) */
+static GFile *trace_dir = NULL;
+
+/* TRUE if tests should be run online and a trace file written for each; FALSE if tests should run offline against existing trace files. */
+static gboolean write_traces = FALSE;
+
+/* TRUE if tests should be run online and the server's responses compared to the existing trace file for each; FALSE if tests should run offline without comparison. */
+static gboolean compare_traces = FALSE;
+
+/* Global mock server instance used by all tests. */
+static GDataMockServer *mock_server = NULL;
+
 void
 gdata_test_init (int argc, char **argv)
 {
@@ -41,16 +60,30 @@ gdata_test_init (int argc, char **argv)
 	g_type_init ();
 #endif
 
-	/* Parse the --no-internet and --no-interactive options */
+	/* Parse the custom options */
 	for (i = 1; i < argc; i++) {
-		if (strcmp ("--no-internet", argv[i]) == 0 || strcmp ("-n", argv[i]) == 0) {
-			no_internet = TRUE;
-			argv[i] = (char*) "";
-		} else if (strcmp ("--no-interactive", argv[i]) == 0 || strcmp ("-i", argv[i]) == 0) {
+		if (strcmp ("--no-interactive", argv[i]) == 0 || strcmp ("-i", argv[i]) == 0) {
 			no_interactive = TRUE;
 			argv[i] = (char*) "";
+		} else if (strcmp ("--trace-dir", argv[i]) == 0 || strcmp ("-t", argv[i]) == 0) {
+			if (i >= argc - 1) {
+				fprintf (stderr, "Error: Missing directory for --trace-dir option.\n");
+				exit (1);
+			}
+
+			trace_dir = g_file_new_for_path (argv[i + 1]);
+
+			argv[i] = (char*) "";
+			argv[i + 1] = (char*) "";
+			i++;
+		} else if (strcmp ("--write-traces", argv[i]) == 0 || strcmp ("-w", argv[i]) == 0) {
+			write_traces = TRUE;
+			argv[i] = (char*) "";
+		} else if (strcmp ("--compare-traces", argv[i]) == 0 || strcmp ("-c", argv[i]) == 0) {
+			compare_traces = TRUE;
+			argv[i] = (char*) "";
 		} else if (strcmp ("-?", argv[i]) == 0 || strcmp ("--help", argv[i]) == 0 || strcmp ("-h" , argv[i]) == 0) {
-			/* We have to override --help in order to document --no-internet and --no-interactive */
+			/* We have to override --help in order to document --no-interactive and the trace flags. */
 			printf ("Usage:\n"
 			          "  %s [OPTION...]\n\n"
 			          "Help Options:\n"
@@ -64,34 +97,53 @@ gdata_test_init (int argc, char **argv)
 			          "  -p TESTPATH                    Execute all tests matching TESTPATH\n"
 			          "  -m {perf|slow|thorough|quick}  Execute tests according modes\n"
 			          "  --debug-log                    Debug test logging output\n"
-			          "  -n, --no-internet              Only execute tests which don't require Internet connectivity\n"
-			          "  -i, --no-interactive           Only execute tests which don't require user interaction\n",
+			          "  -i, --no-interactive           Only execute tests which don't require user interaction\n"
+			          "  -t, --trace-dir [directory]    Read/Write trace files in the specified directory\n"
+			          "  -w, --write-traces             Work online and write trace files to --trace-dir\n"
+			          "  -c, --compare-traces           Work online and compare with existing trace files in --trace-dir\n",
 			          argv[0]);
 			exit (0);
 		}
 	}
 
+	/* --[write|compare]-traces are mutually exclusive. */
+	if (write_traces == TRUE && compare_traces == TRUE) {
+		fprintf (stderr, "Error: --write-traces and --compare-traces are mutually exclusive.\n");
+		exit (1);
+	}
+
 	g_test_init (&argc, &argv, NULL);
 	g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=");
 
-	/* Enable full debugging */
-	g_setenv ("LIBGDATA_DEBUG", "3" /* GDATA_LOG_FULL */, FALSE);
+	/* Set handler of debug information */
+	g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, (GLogFunc) gdata_test_debug_handler, message_list);
+
+	/* Set handler for printerr */
+	g_set_printerr_handler ((GPrintFunc) gdata_test_assert_handler);
+
+	/* Enable full debugging. These options are seriously unsafe, but we don't care for test cases. */
+	g_setenv ("LIBGDATA_DEBUG", "4" /* GDATA_LOG_FULL_UNREDACTED */, FALSE);
 	g_setenv ("G_MESSAGES_DEBUG", "libgdata", FALSE);
+	g_setenv ("LIBGDATA_LAX_SSL_CERTIFICATES", "1", FALSE);
+
+	mock_server = gdata_mock_server_new ();
+	gdata_mock_server_set_enable_logging (mock_server, write_traces);
+	gdata_mock_server_set_enable_online (mock_server, write_traces || compare_traces);
 }
 
 /*
- * gdata_test_internet:
+ * gdata_test_get_mock_server:
  *
- * Returns whether tests which require Internet access should be run.
+ * Returns the singleton #GDataMockServer instance used throughout the test suite.
  *
- * Return value: %TRUE if Internet-requiring tests should be run, %FALSE otherwise
+ * Return value: (transfer none): the mock server
  *
- * Since: 0.7.0
+ * Since: 0.13.4
  */
-gboolean
-gdata_test_internet (void)
+GDataMockServer *
+gdata_test_get_mock_server (void)
 {
-	return (no_internet == FALSE) ? TRUE : FALSE;
+	return mock_server;
 }
 
 /*
@@ -651,3 +703,161 @@ gdata_tear_down_async_test_data (GDataAsyncTestData *async_data, gconstpointer t
 	g_object_unref (async_data->cancellable);
 	g_main_loop_unref (async_data->main_loop);
 }
+
+void
+gdata_test_debug_output (void)
+{
+	if (message_list == NULL) {
+		/* No debugging messages */
+		return;
+	}
+	do {
+		gchar *message = (gchar*) message_list->data;
+		if (message != NULL) {
+			if (strlen (message) > 2 && message[2] == '<') {
+				/* As debug string starts with direction indicator and space, t.i. "< ", */
+				/* we need access string starting from third character and see if it's   */
+				/* looks like xml - t.i. it starts with '<' */
+				xmlChar *xml_buff;
+				int buffer_size;
+				xmlDocPtr xml_doc;
+				/* we need to cut to the begining of XML string */
+				message = message + 2;
+				/* create xml document and dump it to string buffer */
+				xml_doc = xmlParseDoc ((const xmlChar*) message);
+				xmlDocDumpFormatMemory (xml_doc, &xml_buff, &buffer_size, 1);
+				/* print out structured xml - if it's not xml, it will get error in output */
+				printf("%s", (gchar*) xml_buff);
+				/* free xml structs */
+				xmlFree (xml_buff);
+				xmlFreeDoc (xml_doc);
+			} else {
+				printf ("%s\n", (gchar*) message);
+			}
+			/* free gchar */
+			g_free ((gpointer) message_list->data);
+		}
+		/* free GSList node */
+		message_list = g_slist_delete_link (message_list, message_list);
+	} while (message_list != NULL);
+}
+
+static void
+gdata_test_debug_handler (const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
+{
+	/* storing debugging messages in GSList for displaying them in case of error */
+	message_list = g_slist_append (message_list, g_strdup (message));
+
+	/* Log to the trace file. */
+	if (message != NULL && (*message == '<' || *message == '>' || *message == ' ') && *(message + 1) == ' ') {
+		gdata_mock_server_received_message_chunk (mock_server, message, strlen (message));
+	}
+}
+
+static void
+gdata_test_assert_handler (const gchar *message)
+{
+	gdata_test_debug_output ();
+	printf ("%s", (gchar*) message);
+}
+
+/**
+ * gdata_test_set_https_port:
+ * @server: a #GDataMockServer
+ *
+ * Sets the HTTPS port used for all future libgdata requests to that used by the given mock @server,
+ * effectively redirecting all client requests to the mock server.
+ *
+ * Since: 0.13.4
+ */
+void
+gdata_test_set_https_port (GDataMockServer *server)
+{
+	gchar *port_string = g_strdup_printf ("%u", gdata_mock_server_get_port (server));
+	g_setenv ("LIBGDATA_HTTPS_PORT", port_string, TRUE);
+	g_free (port_string);
+}
+
+/**
+ * gdata_test_mock_server_start_trace:
+ * @server: a #GDataMockServer
+ * @trace_filename: filename of the trace to load
+ *
+ * Wrapper around gdata_mock_server_start_trace() which additionally sets the <code class="literal">LIBGDATA_HTTPS_PORT</code>
+ * environment variable to redirect all libgdata requests to the mock server.
+ *
+ * Since: 0.13.4
+ */
+void
+gdata_test_mock_server_start_trace (GDataMockServer *server, const gchar *trace_filename)
+{
+	const gchar *ip_address;
+	GDataMockResolver *resolver;
+
+	gdata_mock_server_start_trace (server, trace_filename);
+	gdata_test_set_https_port (server);
+
+	if (gdata_mock_server_get_enable_online (server) == FALSE) {
+		/* Set up the expected domain names here. This should technically be split up between
+		 * the different unit test suites, but that's too much effort. */
+		ip_address = soup_address_get_physical (gdata_mock_server_get_address (server));
+		resolver = gdata_mock_server_get_resolver (server);
+
+		gdata_mock_resolver_add_A (resolver, "www.google.com", ip_address);
+		gdata_mock_resolver_add_A (resolver, "gdata.youtube.com", ip_address);
+		gdata_mock_resolver_add_A (resolver, "uploads.gdata.youtube.com", ip_address);
+		gdata_mock_resolver_add_A (resolver, "picasaweb.google.com", ip_address);
+		gdata_mock_resolver_add_A (resolver, "docs.google.com", ip_address);
+		gdata_mock_resolver_add_A (resolver, "lh3.googleusercontent.com", ip_address);
+		gdata_mock_resolver_add_A (resolver, "lh5.googleusercontent.com", ip_address);
+		gdata_mock_resolver_add_A (resolver, "lh6.googleusercontent.com", ip_address);
+	}
+}
+
+/**
+ * gdata_test_mock_server_handle_message_error:
+ * @server: a #GDataMockServer
+ * @message: the message whose response should be filled
+ * @client: the currently connected client
+ * @user_data: user data provided when connecting the signal
+ *
+ * Handler for #GDataMockServer::handle-message which sets the HTTP response for @message to the HTTP error status
+ * specified in a #GDataTestRequestErrorData structure passed to @user_data.
+ *
+ * Since: 0.13.4
+ */
+gboolean
+gdata_test_mock_server_handle_message_error (GDataMockServer *server, SoupMessage *message, SoupClientContext *client, gpointer user_data)
+{
+	const GDataTestRequestErrorData *data = user_data;
+
+	soup_message_set_status_full (message, data->status_code, data->reason_phrase);
+	soup_message_body_append (message->response_body, SOUP_MEMORY_STATIC, data->message_body, strlen (data->message_body));
+
+	return TRUE;
+}
+
+/**
+ * gdata_test_mock_server_handle_message_timeout:
+ * @server: a #GDataMockServer
+ * @message: the message whose response should be filled
+ * @client: the currently connected client
+ * @user_data: user data provided when connecting the signal
+ *
+ * Handler for #GDataMockServer::handle-message which waits for 2 seconds before returning a %SOUP_STATUS_REQUEST_TIMEOUT status
+ * and appropriate error message body. If used in conjunction with a 1 second timeout in the client code under test, this can
+ * simulate network error conditions and timeouts, in order to test the error handling code for such conditions.
+ *
+ * Since: 0.13.4
+ */
+gboolean
+gdata_test_mock_server_handle_message_timeout (GDataMockServer *server, SoupMessage *message, SoupClientContext *client, gpointer user_data)
+{
+	/* Sleep for longer than the timeout set on the client. */
+	g_usleep (2 * G_USEC_PER_SEC);
+
+	soup_message_set_status_full (message, SOUP_STATUS_REQUEST_TIMEOUT, "Request Timeout");
+	soup_message_body_append (message->response_body, SOUP_MEMORY_STATIC, "Request timed out.", strlen ("Request timed out."));
+
+	return TRUE;
+}
